use lib qw(/home/goodarzi/usr/lib/perl5/site_perl/5.8.5 /home/goodarzi/usr/lib64/perl5/site_perl/5.8.5/x86_64-linux-thread-multi/auto);

use lib "$ENV{FIREDIR}/SCRIPTS";

use strict ;
use Getopt::Long;
use Sets ;
use Data::Dumper ;
use Statistics::PointEstimation;
use Statistics::TTest;

my $firedir = "$ENV{FIREDIR}";
my $homedir = "$ENV{HOME}";
my @strains = ("MG1655", "AC_000091", "NC_002655", "NC_002951", "NC_003197", "NC_004741", "NC_008563") ;

my $expfile = undef ;

GetOptions('expfile=s'    => \$expfile) ;

open I, "< ../CONSERVATION/bnum_to_gi_translation.txt" ;
my %GI ;
while(<I>){
    chomp;
    my ($bn, $gi) = split(/\t/, $_);
    $GI{$bn} = $gi ;
}

my $fn = Sets::filename($expfile) ;
my $bin_dna = "$expfile\_FIRE/DNA/$fn.binary" ;
my $bin_rna = "$expfile\_FIRE/RNA/$fn.binary" ;

my $pro_dna = "$expfile\_FIRE/DNA/$fn.profiles" ;
my $pro_rna = "$expfile\_FIRE/RNA/$fn.profiles" ;

my @genes ;
print "Reading expression file...";
open I, "< $expfile" or die "Couldn't: $expfile";
<I> ;
while(<I>){
    chomp ;
    my ($tu, $v) = split(/\t/, $_) ;
    $tu =~ /tu_(\S+)_/ ;
    my $b = $1 ;
    push (@genes, $b) if (! grep {$_ eq $b} (@genes)) ;
}
close I ;
print "Done\n";

print "Reading binary files...";
open I, "< $bin_dna" or die "Couldn't: $bin_dna";
my $l = <I>;
chomp $l;
my @motifs_dna = split(/\t/, $l);
shift @motifs_dna;
my %genes_sig_clus_motif ; #The genes that are present in a significant cluster
while(<I>){
    chomp;
    my ($gene, @a) = split(/\t/, $_);
    for (my $i=0 ; $i<@motifs_dna ; $i++){
        push(@{$genes_sig_clus_motif{$motifs_dna[$i]}}, $gene) if ($a[$i]==1);
    }
}
close I;
print "Done\n";

close I ;
open I, "< $bin_rna" or die "Couldn't: $bin_rna";
my $l = <I>;
chomp $l;
my @motifs_rna = split(/\t/, $l);
shift @motifs_rna;
while(<I>){
    chomp;
    my ($gene, @a) = split(/\t/, $_);
    for (my $i=0 ; $i<@motifs_rna ; $i++){
        push(@{$genes_sig_clus_motif{$motifs_rna[$i]}}, $gene) if ($a[$i]==1);
    }
}
close I;
print "Done\n";

my %genes_with_motif; #i.e. genes with the motif that are in a significant cluster
open I, "< $pro_dna" or die "Couldn't: $pro_dna";
print "Reading motif profiles...";
while(<I>){
    chomp;
    my ($m, $g, @a) = split(/\t/, $_);
    next if (! defined $genes_sig_clus_motif{$m});
    my @a = @{$genes_sig_clus_motif{$m}} ;
    if (grep {$g eq $_} (@a)){
	$g =~ /tu_(\S+)_/ ;
	my $b = $1 ;
        push(@{$genes_with_motif{$m}}, $b);
    }
}
close I;

open I, "< $pro_rna" or die "Couldn't: $pro_rna";
while(<I>){
    chomp;
    my ($m, $g, @a) = split(/\t/, $_);
    next if (! defined $genes_sig_clus_motif{$m});
    my @a = @{$genes_sig_clus_motif{$m}} ;
    if (grep {$g eq $_} (@a)){
	$g =~ /tu_(\S+)_/ ;
        my $b = $1 ;
        push(@{$genes_with_motif{$m}}, $g);
    }
}
close I;
print "Done\n";

my %HASH ;
my %UPSEQ ;
my %DOWNSEQ ;
foreach my $s (@strains){
    open I, "< ../CONSERVATION/$s\_upstream_200bp.txt" or die "../CONSERVATION/$s\_upstream_200bp.txt";
    while(<I>){
	chomp ;
	my $gi = substr($_, 1) ;
	my $seq = <I> ;
	chomp $seq ;
	$UPSEQ{$s}->{$gi} = $seq ;
    }
    close I ;
    open I, "< ../CONSERVATION/$s\_downstream_200bp.txt" or die "../CONSERVATION/$s\_downstream_200bp.txt";
    while(<I>){
        chomp ;
        my $gi = substr($_, 1) ;
        my $seq = <I> ;
        chomp $seq ;
        $DOWNSEQ{$s}->{$gi} = $seq ;
    }
    close I ;
    next if ($s eq "MG1655") ;
    open I, "< ../CONSERVATION/NC_000913vs$s.rbr" or die "../CONSERVATION/NC_000913vs$s.rbr";
    while(<I>){
        chomp;
        my ($gi1, $gi2) = split(/\t/, $_) ;
        $HASH{$s}->{$gi1} = $gi2 ;
    }
    close I ;
}

my @motifs = (@motifs_dna, @motifs_rna);
my %CONS ;
foreach my $motif (@motifs){
    my $rna = 0 ;
    $rna = 1 if (grep {$_ eq $motif} (@motifs_rna)) ;
    my @score ;
    my @count ;
    my $c0 = 0 ;
    my $c1 = 0 ;
    for (my $i=0 ; $i<@genes ; $i++){
	my $b = $genes[$i] ;
	my $gi = $GI{$b} ;
	my $seq ;
	if ($rna){
	    $seq = $DOWNSEQ{"MG1655"}->{$gi} ;
	}
	else{
	    $seq = $UPSEQ{"MG1655"}->{$gi} ;
	}
	next if (! (Sets::getREMotifPositions($motif, $seq))) ;
	my $category = 0 ;
	$category = 1 if (grep {$_ eq $b} (@{$genes_with_motif{$motif}})) ;

	foreach my $s (@strains){
	    next if ($s eq "MG1655") ;
	    my $gi2 = $HASH{$s}->{$gi} ;
	    next if (! defined $gi2) ;
	    my $seq ;
	    if ($rna){
		$seq = $DOWNSEQ{$s}->{$gi2} ;
	    }
	    else{
		$seq = $UPSEQ{$s}->{$gi2} ;
	    }
	    $count[$category]->[$c0]++ if ($category==0);
	    $count[$category]->[$c1]++ if ($category==1);
	    if ($seq =~ /$motif/){
		$score[$category]->[$c0]++ if ($category==0);
		$score[$category]->[$c1]++ if ($category==1);
	    }
	}
	$c0++ if ($category==0 and $count[0]->[$c0]>0) ;
	$c1++ if ($category==1 and $count[1]->[$c1]>0) ;
    }
    next if ($c1<3 or $c0<3) ;
    print "c0 is $c0 and c1 is $c1\n" ;

    for (my $i=0 ; $i<$c0 ; $i++){
	if ($count[0]->[$i]!=0){
	    $score[0]->[$i]/=$count[0]->[$i] ;
	    $score[0]->[$i] = log($score[0]->[$i]+0.0001) ;
	}
    }
    for (my $i=0 ; $i<$c1 ; $i++){
	if ($count[1]->[$i]!=0){
            $score[1]->[$i]/=$count[1]->[$i] ;
            $score[1]->[$i] = log($score[1]->[$i]+0.0001) ;
        }
    }

    my $ttest = new Statistics::TTest;  
    $ttest->set_significance(95);
    $ttest->load_data($score[1],$score[0]);
    my $p = $ttest->t_prob ;


    print "Motif\tCategory 1\tCategory 2\tRatio\n" ;
    print $motif, "\t", Sets::average($score[1]), "\t", Sets::average($score[0]), "\t", $p,"\n" ;
}
